@@ -17,23 +17,16 @@
 import copy
 import uuid
 
-from keystone import exception
 from keystone.common import logging
 from keystone.common import wsgi
-from keystone.identity import Manager as IdentityManager
-from keystone.identity import UserController as UserManager
-from keystone.token import Manager as TokenManager
+from keystone import exception
+from keystone import identity
 
 
 LOG = logging.getLogger(__name__)
 
 
-class UserController(wsgi.Application):
-    def __init__(self):
-        self.identity_api = IdentityManager()
-        self.token_api = TokenManager()
-        self.user_controller = UserManager()
-
+class UserController(identity.controllers.User):
     def set_user_password(self, context, user_id, user):
         token_id = context.get('token_id')
         original_password = user.get('original_password')
@@ -42,8 +35,11 @@ class UserController(wsgi.Application):
                                              token_id=token_id)
         user_id_from_token = token_ref['user']['id']
 
-        if user_id_from_token != user_id or original_password is None:
-            raise exception.Forbidden()
+        if user_id_from_token != user_id:
+            raise exception.Forbidden('Token belongs to another user')
+        if original_password is None:
+            raise exception.ValidationError(target='user',
+                                            attribute='original password')
 
         try:
             user_ref = self.identity_api.authenticate(
@@ -51,7 +47,8 @@ class UserController(wsgi.Application):
                 user_id=user_id_from_token,
                 password=original_password)[0]
             if not user_ref.get('enabled', True):
-                raise exception.Unauthorized()
+                # NOTE(dolph): why can't you set a disabled user's password?
+                raise exception.Unauthorized('User is disabled')
         except AssertionError:
             raise exception.Unauthorized()
 
@@ -59,9 +56,9 @@ class UserController(wsgi.Application):
 
         admin_context = copy.copy(context)
         admin_context['is_admin'] = True
-        self.user_controller.set_user_password(admin_context,
-                                               user_id,
-                                               update_dict)
+        super(UserController, self).set_user_password(admin_context,
+                                                      user_id,
+                                                      update_dict)
 
         token_id = uuid.uuid4().hex
         new_token_ref = copy.copy(token_ref)
