@@ -962,14 +962,29 @@ class Identity(sql.Base, identity.Driver):
 
         with session.begin():
             for metadata_ref in session.query(UserProjectGrant):
-                metadata = metadata_ref.to_dict()
                 try:
-                    self.remove_role_from_user_and_project(
-                        metadata['user_id'], metadata['project_id'], role_id)
+                    self.delete_grant(role_id, user_id=metadata_ref.user_id,
+                                      project_id=metadata_ref.project_id)
+                except exception.RoleNotFound:
+                    pass
+            for metadata_ref in session.query(UserDomainGrant):
+                try:
+                    self.delete_grant(role_id, user_id=metadata_ref.user_id,
+                                      domain_id=metadata_ref.domain_id)
+                except exception.RoleNotFound:
+                    pass
+            for metadata_ref in session.query(GroupProjectGrant):
+                try:
+                    self.delete_grant(role_id, group_id=metadata_ref.group_id,
+                                      project_id=metadata_ref.project_id)
+                except exception.RoleNotFound:
+                    pass
+            for metadata_ref in session.query(GroupDomainGrant):
+                try:
+                    self.delete_grant(role_id, group_id=metadata_ref.group_id,
+                                      domain_id=metadata_ref.domain_id)
                 except exception.RoleNotFound:
                     pass
-
-            # FIXME(dolph): user-domain metadata needs to be updated
 
             if not session.query(Role).filter_by(id=role_id).delete():
                 raise exception.RoleNotFound(role_id=role_id)
