@@ -20,12 +20,15 @@ import webob
 
 import nose.exc
 
-from keystone import test
 from keystone.openstack.common import jsonutils
-
+from keystone.openstack.common import timeutils
+from keystone import config
+from keystone import test
 
 import default_fixtures
 
+CONF = config.CONF
+DEFAULT_DOMAIN_ID = CONF.identity.default_domain_id
 OPENSTACK_REPO = 'https://review.openstack.org/p/openstack'
 KEYSTONECLIENT_REPO = '%s/python-keystoneclient.git' % OPENSTACK_REPO
 
@@ -50,7 +53,7 @@ class CompatTestCase(test.TestCase):
         # override the fixtures, for now
         self.metadata_foobar = self.identity_api.update_metadata(
             self.user_foo['id'], self.tenant_bar['id'],
-            dict(roles=['keystone_admin'], is_admin='1'))
+            dict(roles=[self.role_admin['id']], is_admin='1'))
 
     def tearDown(self):
         self.public_server.kill()
@@ -140,6 +143,14 @@ class KeystoneClientTests(object):
                           self._client, token=token,
                           tenant_id=uuid.uuid4().hex)
 
+    def test_authenticate_token_invalid_tenant_name(self):
+        from keystoneclient import exceptions as client_exceptions
+        client = self.get_client()
+        token = client.auth_token
+        self.assertRaises(client_exceptions.Unauthorized,
+                          self._client, token=token,
+                          tenant_name=uuid.uuid4().hex)
+
     def test_authenticate_token_tenant_name(self):
         client = self.get_client()
         token = client.auth_token
@@ -385,16 +396,41 @@ class KeystoneClientTests(object):
                           self.get_client,
                           self.user_foo)
 
+    def test_delete_user_invalidates_token(self):
+        from keystoneclient import exceptions as client_exceptions
+
+        admin_client = self.get_client(admin=True)
+        client = self.get_client(admin=False)
+
+        username = uuid.uuid4().hex
+        password = uuid.uuid4().hex
+        user_id = admin_client.users.create(
+            name=username, password=password, email=uuid.uuid4().hex).id
+
+        token_id = client.tokens.authenticate(
+            username=username, password=password).id
+
+        # token should be usable before the user is deleted
+        client.tokens.authenticate(token=token_id)
+
+        admin_client.users.delete(user=user_id)
+
+        # authenticate with a token should not work after the user is deleted
+        self.assertRaises(client_exceptions.Unauthorized,
+                          client.tokens.authenticate,
+                          token=token_id)
+
     def test_token_expiry_maintained(self):
         foo_client = self.get_client(self.user_foo)
-        orig_token = foo_client.service_catalog.catalog['token']
 
-        time.sleep(1.01)
+        orig_token = foo_client.service_catalog.catalog['token']
+        time.sleep(.5)
         reauthenticated_token = foo_client.tokens.authenticate(
             token=foo_client.auth_token)
 
-        self.assertEquals(orig_token['expires'],
-                          reauthenticated_token.expires)
+        self.assertCloseEnoughForGovernmentWork(
+            timeutils.parse_isotime(orig_token['expires']),
+            timeutils.parse_isotime(reauthenticated_token.expires))
 
     def test_user_create_update_delete(self):
         from keystoneclient import exceptions as client_exceptions
@@ -524,8 +560,8 @@ class KeystoneClientTests(object):
 
     def test_role_get(self):
         client = self.get_client(admin=True)
-        role = client.roles.get(role='keystone_admin')
-        self.assertEquals(role.id, 'keystone_admin')
+        role = client.roles.get(role=self.role_admin['id'])
+        self.assertEquals(role.id, self.role_admin['id'])
 
     def test_role_crud(self):
         from keystoneclient import exceptions as client_exceptions
@@ -701,7 +737,7 @@ class KeystoneClientTests(object):
         self.assertEquals(service_type, service.type)
         self.assertEquals(service_desc, service.description)
 
-        # update is not supported...
+        # update is not supported in API v2...
 
         # delete & read
         client.services.delete(id=service.id)
@@ -726,10 +762,9 @@ class KeystoneClientTests(object):
                           id=uuid.uuid4().hex)
 
     def test_endpoint_delete_404(self):
-        # the catalog backend is expected to return Not Implemented
         from keystoneclient import exceptions as client_exceptions
         client = self.get_client(admin=True)
-        self.assertRaises(client_exceptions.HTTPNotImplemented,
+        self.assertRaises(client_exceptions.NotFound,
                           client.endpoints.delete,
                           id=uuid.uuid4().hex)
 
@@ -773,7 +808,7 @@ class KeystoneClientTests(object):
         # ROLE CRUD
         self.assertRaises(exception,
                           two.roles.get,
-                          role='keystone_admin')
+                          role=self.role_admin['id'])
         self.assertRaises(exception,
                           two.roles.list)
         self.assertRaises(exception,
@@ -781,7 +816,7 @@ class KeystoneClientTests(object):
                           name='oops')
         self.assertRaises(exception,
                           two.roles.delete,
-                          role='keystone_admin')
+                          role=self.role_admin['id'])
 
         # TODO(ja): MEMBERSHIP CRUD
         # TODO(ja): determine what else todo
@@ -789,20 +824,23 @@ class KeystoneClientTests(object):
 
 class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
     def get_checkout(self):
-        return KEYSTONECLIENT_REPO, '0.2.0'
+        return KEYSTONECLIENT_REPO, 'master'
 
     def test_tenant_add_and_remove_user(self):
         client = self.get_client(admin=True)
-        client.roles.add_user_role(tenant=self.tenant_baz['id'],
+        client.roles.add_user_role(tenant=self.tenant_bar['id'],
                                    user=self.user_two['id'],
-                                   role=self.role_useless['id'])
-        user_refs = client.tenants.list_users(tenant=self.tenant_baz['id'])
+                                   role=self.role_other['id'])
+        user_refs = client.tenants.list_users(tenant=self.tenant_bar['id'])
         self.assert_(self.user_two['id'] in [x.id for x in user_refs])
-        client.roles.remove_user_role(tenant=self.tenant_baz['id'],
+        client.roles.remove_user_role(tenant=self.tenant_bar['id'],
                                       user=self.user_two['id'],
-                                      role=self.role_useless['id'])
-        user_refs = client.tenants.list_users(tenant=self.tenant_baz['id'])
-        self.assert_(self.user_two['id'] not in [x.id for x in user_refs])
+                                      role=self.role_other['id'])
+        roles = client.roles.roles_for_user(user=self.user_foo['id'],
+                                            tenant=self.tenant_bar['id'])
+        self.assertNotIn(self.role_other['id'], roles)
+        user_refs = client.tenants.list_users(tenant=self.tenant_bar['id'])
+        self.assertNotIn(self.user_two['id'], [x.id for x in user_refs])
 
     def test_user_role_add_404(self):
         from keystoneclient import exceptions as client_exceptions
@@ -811,12 +849,12 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
                           client.roles.add_user_role,
                           tenant=uuid.uuid4().hex,
                           user=self.user_foo['id'],
-                          role=self.role_useless['id'])
+                          role=self.role_member['id'])
         self.assertRaises(client_exceptions.NotFound,
                           client.roles.add_user_role,
                           tenant=self.tenant_baz['id'],
                           user=uuid.uuid4().hex,
-                          role=self.role_useless['id'])
+                          role=self.role_member['id'])
         self.assertRaises(client_exceptions.NotFound,
                           client.roles.add_user_role,
                           tenant=self.tenant_baz['id'],
@@ -830,12 +868,12 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
                           client.roles.remove_user_role,
                           tenant=uuid.uuid4().hex,
                           user=self.user_foo['id'],
-                          role=self.role_useless['id'])
+                          role=self.role_member['id'])
         self.assertRaises(client_exceptions.NotFound,
                           client.roles.remove_user_role,
                           tenant=self.tenant_baz['id'],
                           user=uuid.uuid4().hex,
-                          role=self.role_useless['id'])
+                          role=self.role_member['id'])
         self.assertRaises(client_exceptions.NotFound,
                           client.roles.remove_user_role,
                           tenant=self.tenant_baz['id'],
@@ -845,7 +883,7 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
                           client.roles.remove_user_role,
                           tenant=self.tenant_baz['id'],
                           user=self.user_foo['id'],
-                          role=self.role_useless['id'])
+                          role=self.role_member['id'])
 
     def test_tenant_list_marker(self):
         client = self.get_client()
@@ -853,10 +891,11 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
         # Add two arbitrary tenants to user for testing purposes
         for i in range(2):
             tenant_id = uuid.uuid4().hex
-            tenant = {'name': 'tenant-%s' % tenant_id, 'id': tenant_id}
-            self.identity_api.create_tenant(tenant_id, tenant)
-            self.identity_api.add_user_to_tenant(tenant_id,
-                                                 self.user_foo['id'])
+            tenant = {'name': 'tenant-%s' % tenant_id, 'id': tenant_id,
+                      'domain_id': DEFAULT_DOMAIN_ID}
+            self.identity_api.create_project(tenant_id, tenant)
+            self.identity_api.add_user_to_project(tenant_id,
+                                                  self.user_foo['id'])
 
         tenants = client.tenants.list()
         self.assertEqual(len(tenants), 3)
@@ -879,10 +918,11 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
         # Add two arbitrary tenants to user for testing purposes
         for i in range(2):
             tenant_id = uuid.uuid4().hex
-            tenant = {'name': 'tenant-%s' % tenant_id, 'id': tenant_id}
-            self.identity_api.create_tenant(tenant_id, tenant)
-            self.identity_api.add_user_to_tenant(tenant_id,
-                                                 self.user_foo['id'])
+            tenant = {'name': 'tenant-%s' % tenant_id, 'id': tenant_id,
+                      'domain_id': DEFAULT_DOMAIN_ID}
+            self.identity_api.create_project(tenant_id, tenant)
+            self.identity_api.add_user_to_project(tenant_id,
+                                                  self.user_foo['id'])
 
         tenants = client.tenants.list()
         self.assertEqual(len(tenants), 3)
@@ -924,15 +964,15 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
             '/v2.0/OS-KSCRUD/users/%s' % self.user_two['id'],
             headers={'X-Auth-Token': token_id})
         req.method = 'PATCH'
-        req.body = '{"user":{"password":"%s","original_password":"%s"}}' % \
-            (new_password, self.user_two['password'])
+        req.body = ('{"user":{"password":"%s","original_password":"%s"}}' %
+                    (new_password, self.user_two['password']))
         self.public_server.application(req.environ,
                                        responseobject.start_fake_response)
 
         self.user_two['password'] = new_password
         self.get_client(self.user_two)
 
-    def test_user_cant_update_other_users_passwd(self):
+    def test_user_cannot_update_other_users_passwd(self):
         from keystoneclient import exceptions as client_exceptions
 
         client = self.get_client(self.user_two)
@@ -951,8 +991,8 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
             '/v2.0/OS-KSCRUD/users/%s' % self.user_foo['id'],
             headers={'X-Auth-Token': token_id})
         req.method = 'PATCH'
-        req.body = '{"user":{"password":"%s","original_password":"%s"}}' % \
-            (new_password, self.user_two['password'])
+        req.body = ('{"user":{"password":"%s","original_password":"%s"}}' %
+                    (new_password, self.user_two['password']))
         self.public_server.application(req.environ,
                                        responseobject.start_fake_response)
         self.assertEquals(403, responseobject.response_status)
@@ -980,14 +1020,14 @@ class KcMasterTestCase(CompatTestCase, KeystoneClientTests):
             '/v2.0/OS-KSCRUD/users/%s' % self.user_two['id'],
             headers={'X-Auth-Token': token_id})
         req.method = 'PATCH'
-        req.body = '{"user":{"password":"%s","original_password":"%s"}}' % \
-            (new_password, self.user_two['password'])
+        req.body = ('{"user":{"password":"%s","original_password":"%s"}}' %
+                    (new_password, self.user_two['password']))
 
         rv = self.public_server.application(
             req.environ,
             responseobject.start_fake_response)
-        responce_json = jsonutils.loads(rv.next())
-        new_token_id = responce_json['access']['token']['id']
+        response_json = jsonutils.loads(rv.pop())
+        new_token_id = response_json['access']['token']['id']
 
         self.assertRaises(client_exceptions.Unauthorized, client.tenants.list)
         client.auth_token = new_token_id
@@ -1000,9 +1040,9 @@ class KcEssex3TestCase(CompatTestCase, KeystoneClientTests):
 
     def test_tenant_add_and_remove_user(self):
         client = self.get_client(admin=True)
-        client.roles.add_user_to_tenant(tenant_id=self.tenant_baz['id'],
+        client.roles.add_user_to_tenant(tenant_id=self.tenant_bar['id'],
                                         user_id=self.user_two['id'],
-                                        role_id=self.role_useless['id'])
+                                        role_id=self.role_member['id'])
         role_refs = client.roles.get_user_role_refs(
             user_id=self.user_two['id'])
         self.assert_(self.tenant_baz['id'] in [x.tenantId for x in role_refs])
@@ -1012,12 +1052,12 @@ class KcEssex3TestCase(CompatTestCase, KeystoneClientTests):
         roleref_refs = client.roles.get_user_role_refs(
             user_id=self.user_two['id'])
         for roleref_ref in roleref_refs:
-            if (roleref_ref.roleId == self.role_useless['id']
+            if (roleref_ref.roleId == self.role_member['id']
                     and roleref_ref.tenantId == self.tenant_baz['id']):
                 # use python's scope fall through to leave roleref_ref set
                 break
 
-        client.roles.remove_user_from_tenant(tenant_id=self.tenant_baz['id'],
+        client.roles.remove_user_from_tenant(tenant_id=self.tenant_bar['id'],
                                              user_id=self.user_two['id'],
                                              role_id=roleref_ref.id)
 
@@ -1087,7 +1127,14 @@ class KcEssex3TestCase(CompatTestCase, KeystoneClientTests):
     def test_endpoint_delete_404(self):
         raise nose.exc.SkipTest('N/A')
 
+    def test_policy_crud(self):
+        """Due to lack of endpoint CRUD"""
+        raise nose.exc.SkipTest('N/A')
+
 
 class Kc11TestCase(CompatTestCase, KeystoneClientTests):
     def get_checkout(self):
         return KEYSTONECLIENT_REPO, '0.1.1'
+
+    def test_policy_crud(self):
+        raise nose.exc.SkipTest('N/A')
